#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>

char *RND_name; // Generator name returned by: extern "C" __declspec(dllexport) char *RND_GetName(void)
void SetRNDname(const char *name);


BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{

	// When the system calls the DllMain function with the DLL_PROCESS_ATTACH value,
	// the function returns TRUE if it succeeds or FALSE if initialization fails.
	// If the return value is FALSE when DllMain is called because the process uses the LoadLibrary function, LoadLibrary returns NULL.
	// (The system immediately calls your entry-point function with DLL_PROCESS_DETACH and unloads the DLL.)
	// If the return value is FALSE when DllMain is called during process initialization, the process terminates with an error.
	//
	// When the system calls the DllMain function with any value other than DLL_PROCESS_ATTACH, the return value is ignored.

	switch (ul_reason_for_call)
	{
	// The DLL is being loaded into the virtual address space of the current process as a result of the process
	// starting up or as a result of a call to LoadLibrary. DLLs can use this opportunity to initialize any instance
	//	data or to use the TlsAlloc function to allocate a thread local storage (TLS) index.
	case DLL_PROCESS_ATTACH:

		RND_name = NULL;

		break;

	// The current process is creating a new thread. When this occurs, the system calls the entry-point function of all DLLs currently
	// attached to the process. The call is made in the context of the new thread. DLLs can use this opportunity to initialize a TLS slot
	//	for the thread. A thread calling the DLL entry-point function with DLL_PROCESS_ATTACH does not call the DLL entry-point function
	//	with DLL_THREAD_ATTACH. 
	// Note that a DLL's entry-point function is called with this value only by threads created after the DLL is loaded by the process.
	// When a DLL is loaded using LoadLibrary, existing threads do not call the entry-point function of the newly loaded DLL.
	case DLL_THREAD_ATTACH:

		break;

	// A thread is exiting cleanly. If the DLL has stored a pointer to allocated memory in a TLS slot, it should use this opportunity
	//	to free the memory. The system calls the entry-point function of all currently loaded DLLs with this value. The call is made in
	//	the context of the exiting thread.
	case DLL_THREAD_DETACH:

		break;

	// The DLL is being unloaded from the virtual address space of the calling process as a result of unsuccessfully loading the DLL,
	//	termination of the process, or a call to FreeLibrary. The DLL can use this opportunity to call the TlsFree function to free
	//	any TLS indices allocated by using TlsAlloc and to free any thread local data. 
	// Note that the thread that receives the DLL_PROCESS_DETACH notification is not necessarily the same thread that received the
	//	DLL_PROCESS_ATTACH notification
	case DLL_PROCESS_DETACH:

		delete[] RND_name;

		break;
	}
	return TRUE;
}


// ===============================================
// ===   The RND_GetUL function is mandatory   ===
// ===============================================

extern "C" __declspec(dllexport) ULONG RND_GetUL(void)
{
	static ULONG y= GetTickCount(), flg= 1;
	if(flg) {
		SetRNDname("LCG"); flg= 0;
	}
	y= y * 2147001325L + 715136305L;
	return y;
}


// ====================================================
// ===   The following code can be safely deleted   ===
// ====================================================


// When NULL is returned, RaBiGeTe will not write the name and the description in the output file
extern "C" __declspec(dllexport) char *RND_GetName(void)
{
	return RND_name;
}

// Return the description of the generator
extern "C" __declspec(dllexport) char *RND_GetDesc(void)
{
	return "Description of the generator.";
}

// Return the name of the output file (default: "_RaBiGeTe.txt")
extern "C" __declspec(dllexport) char *RND_GetOutFileName(void)
{
	return "_RaBiGeTe.txt";
}

// Sample function to create RND_name
// You can call this function 'n' times to cancatenate 'n' generators name
void SetRNDname(const char *name)
{
	int l=0; if(RND_name) l=strlen(RND_name);
	l += strlen(name); char *t = new char[l+3]; // Concatenate the RNG names
	if(RND_name) sprintf(t, "%s, %s", RND_name, name); else strcpy(t, name);
	delete[] RND_name; RND_name = t;
}
